@@ -1,7 +1,8 @@
-from contextlib import contextmanager
 import os
 import tempfile
 
+from contextlib import contextmanager
+
 import intake
 import numpy as np
 import pytest
@@ -10,7 +11,7 @@ import yaml
 from bluesky.tests.conftest import RE  # noqa
 from ophyd.tests.conftest import hw  # noqa
 
-from bluesky_kafka import Publisher
+from bluesky_kafka import BlueskyConsumer, Publisher
 from bluesky_kafka.utils import create_topics, delete_topics
 
 
@@ -48,7 +49,17 @@ def kafka_bootstrap_servers(request):
 
 
 @pytest.fixture(scope="function")
-def temporary_topics(kafka_bootstrap_servers):
+def broker_authorization_config():
+    return {
+        # "security.protocol": "SASL_PLAINTEXT",
+        # "sasl.mechanisms": "PLAIN",
+        # "sasl.username": "user",
+        # "sasl.password": "password",
+    }
+
+
+@pytest.fixture(scope="function")
+def temporary_topics(kafka_bootstrap_servers, broker_authorization_config):
     """
     Use this "factory as a fixture and context manager" to cleanly
     create new topics and delete them after a test.
@@ -60,36 +71,46 @@ def temporary_topics(kafka_bootstrap_servers):
     ----------
     kafka_bootstrap_servers : pytest fixture
         comma-delimited str of Kafka bootstrap server host:port specified on the pytest command line
-
+    broker_authorization_config: dict
+        Kafka broker authentication parameters for the test broker
     """
 
     @contextmanager
-    def _temporary_topics(topics, bootstrap_servers=None):
+    def _temporary_topics(topics, bootstrap_servers=None, admin_client_config=None):
         if bootstrap_servers is None:
             bootstrap_servers = kafka_bootstrap_servers
 
+        if admin_client_config is None:
+            admin_client_config = broker_authorization_config
+
         try:
             # delete existing requested topics
             # this will delete any un-consumed messages
             # the intention is to make tests repeatable by ensuring
             # they always start with a topics having no "old" messages
             delete_topics(
-                bootstrap_servers=bootstrap_servers, topics_to_delete=topics
+                bootstrap_servers=bootstrap_servers,
+                topics_to_delete=topics,
+                admin_client_config=admin_client_config,
             )
             create_topics(
-                bootstrap_servers=bootstrap_servers, topics_to_create=topics
+                bootstrap_servers=bootstrap_servers,
+                topics_to_create=topics,
+                admin_client_config=admin_client_config,
             )
             yield topics
         finally:
             delete_topics(
-                bootstrap_servers=bootstrap_servers, topics_to_delete=topics
+                bootstrap_servers=bootstrap_servers,
+                topics_to_delete=topics,
+                admin_client_config=admin_client_config,
             )
 
     return _temporary_topics
 
 
 @pytest.fixture(scope="function")
-def publisher_factory(kafka_bootstrap_servers):
+def publisher_factory(kafka_bootstrap_servers, broker_authorization_config):
     """
     Use this "factory as a fixture" to create one or more Publishers in a test function.
     If `bootstrap_servers` is not specified to the factory function then the `kafka_bootstrap_servers`
@@ -106,6 +127,8 @@ def publisher_factory(kafka_bootstrap_servers):
     ----------
     kafka_bootstrap_servers : pytest fixture
         comma-delimited str of Kafka bootstrap server host:port specified on the pytest command line
+    broker_authorization_config: dict
+        Kafka broker authentication parameters for the test broker
 
     Returns
     -------
@@ -154,6 +177,7 @@ def publisher_factory(kafka_bootstrap_servers):
                 "enable.idempotence": False,
                 "request.timeout.ms": 1000,
             }
+            producer_config.update(broker_authorization_config)
 
         return Publisher(
             topic=topic,
@@ -167,33 +191,148 @@ def publisher_factory(kafka_bootstrap_servers):
 
 
 @pytest.fixture(scope="function")
-def publisher(request, kafka_bootstrap_servers):
+def consume_documents_from_kafka_until_first_stop_document(
+    kafka_bootstrap_servers, broker_authorization_config
+):
+    """Use this fixture to consume Kafka messages containing bluesky (name, document) tuples.
+
+    This fixture will construct a BlueskyConsumer and run its polling loop. When the first
+    stop document is encountered the consumer polling loop will terminate so the test function
+    can continue.
+
+    Parameters
+    ----------
+    kafka_bootstrap_servers : pytest fixture
+        comma-delimited str of Kafka bootstrap server host:port specified on the pytest command line
+    broker_authorization_config: dict
+        Kafka broker authentication parameters for the test broker
+
+    Returns
+    -------
+    _consume_documents_from_kafka: function(topic, bootstrap_servers=None, **bluesky_consumer_kwargs) -> List[(name, document)]
+        calling this function will consume Kafka messages and place the (name, document)
+        tuples into a list; when the first stop document is encountered the consumer
+        polling loop will terminate and the document list will be returned
+    """
+
+    def _consume_documents_from_kafka(
+        kafka_topic,
+        bootstrap_servers=None,
+        consumer_config=None,
+        **bluesky_consumer_kwargs,
+    ):
+        """
+        Parameters
+        ----------
+        kafka_topic: str
+            Kafka messages with this topic will be consumed
+        bootstrap_servers: str, optional
+            Comma-delimited list of Kafka server addresses as a string such as ``'127.0.0.1:9092'``;
+            default is the value of the pytest command line parameter --kafka-bootstrap-servers
+        consumer_config: dict, optional
+            Dictionary of Kafka consumer configuration parameters
+        bluesky_consumer_kwargs:
+            Allows polling_duration and deserializer to be passed the the BlueskyConsumer
+
+        Returns
+        -------
+         consumed_bluesky_documents: list
+             list of (name, document) tuples delivered by Kafka
+        """
+        if bootstrap_servers is None:
+            bootstrap_servers = kafka_bootstrap_servers
+
+        if consumer_config is None:
+            consumer_config = {
+                # this consumer is intended to read messages that
+                # have already been published, so it is necessary
+                # to specify "earliest" here
+                "auto.offset.reset": "earliest",
+            }
+            consumer_config.update(broker_authorization_config)
+
+        consumed_bluesky_documents = []
+
+        def store_consumed_document(consumer, topic, name, document):
+            """This function appends to a list all documents received by the consumer.
+
+            Parameters
+            ----------
+            consumer: bluesky_kafka.BlueskyConsumer
+                unused
+            topic: str
+                unused
+            name: str
+                bluesky document name, such as "start", "descriptor", "event", etc
+            document: dict
+                dictionary of bluesky document data
+            """
+            consumed_bluesky_documents.append((name, document))
+
+        bluesky_consumer = BlueskyConsumer(
+            topics=[kafka_topic],
+            bootstrap_servers=bootstrap_servers,
+            group_id=f"{kafka_topic}.consumer.group",
+            consumer_config=consumer_config,
+            process_document=store_consumed_document,
+            **bluesky_consumer_kwargs,
+        )
+
+        def until_first_stop_document():
+            """
+            This function returns False to end the BlueskyConsumer polling loop after seeing
+            a "stop" document. Without something like this the polling loop will never end.
+            """
+            if "stop" in [name for name, _ in consumed_bluesky_documents]:
+                return False
+            else:
+                return True
+
+        # start() will return when 'until_first_stop_document' returns False
+        bluesky_consumer.start(
+            continue_polling=until_first_stop_document,
+        )
+
+        return consumed_bluesky_documents
+
+    return _consume_documents_from_kafka
+
+
+@pytest.fixture(scope="function")
+def publisher(request, kafka_bootstrap_servers, broker_authorization_config):
+    # work with a single broker
+    producer_config = {
+        "acks": 1,
+        "enable.idempotence": False,
+        "request.timeout.ms": 5000,
+    }
+    producer_config.update(broker_authorization_config)
+
     return Publisher(
         topic=TEST_TOPIC,
         bootstrap_servers=kafka_bootstrap_servers,
         key="kafka-unit-test-key",
-        # work with a single broker
-        producer_config={
-            "acks": 1,
-            "enable.idempotence": False,
-            "request.timeout.ms": 5000,
-        },
+        producer_config=producer_config,
         flush_on_stop_doc=True,
     )
 
 
 @pytest.fixture(scope="function")
-def publisher2(request, kafka_bootstrap_servers):
+def publisher2(request, kafka_bootstrap_servers, broker_authorization_config):
+    # work with a single broker
+    producer_config = {
+        "acks": 1,
+        "enable.idempotence": False,
+        "request.timeout.ms": 5000,
+    }
+    producer_config.update(broker_authorization_config)
+
     return Publisher(
         topic=TEST_TOPIC2,
         bootstrap_servers=kafka_bootstrap_servers,
         key="kafka-unit-test-key",
         # work with a single broker
-        producer_config={
-            "acks": 1,
-            "enable.idempotence": False,
-            "request.timeout.ms": 5000,
-        },
+        producer_config=producer_config,
         flush_on_stop_doc=True,
     )
 
